@@ -426,7 +426,7 @@ void CrolPlayer::send_operator( int const voice, SOPL2Op const &modulator,  SOPL
         opl->write( 0xc0+voice    , modulator.fbc      );
         opl->write( 0xe0+op_offset, modulator.waveform );
 
-        volumeCache[voice] = (carrier.ksltl & 0xc0) | volumeCache[voice] & 0x3f;
+        volumeCache[voice] = (carrier.ksltl & 0xc0) | (volumeCache[voice] & 0x3f);
 
         opl->write( 0x23+op_offset, carrier.ammulti  );
         opl->write( 0x43+op_offset, volumeCache[voice]    );
@@ -439,7 +439,7 @@ void CrolPlayer::send_operator( int const voice, SOPL2Op const &modulator,  SOPL
     {
         int const op_offset = drum_table[voice-kSnareDrumChannel];
 
-        volumeCache[voice] = (modulator.ksltl & 0xc0) | volumeCache[voice] & 0x3f;
+        volumeCache[voice] = (modulator.ksltl & 0xc0) | (volumeCache[voice] & 0x3f);
 
         opl->write( 0x20+op_offset, modulator.ammulti  );
         opl->write( 0x40+op_offset, volumeCache[voice]      );
@@ -469,7 +469,7 @@ void CrolPlayer::load_tempo_events( binistream *f )
 bool CrolPlayer::load_voice_data( binistream *f, std::string const &bnk_filename, const CFileProvider &fp )
 {
     SBnkHeader bnk_header;
-    VFSFile *fd = aud_vfs_fopen(bnk_filename.c_str(), "rb");
+    VFSFile *fd = vfs_fopen(bnk_filename.c_str(), "rb");
     binistream *bnk_file = fp.open(fd);
 
     if( bnk_file )
@@ -492,7 +492,7 @@ bool CrolPlayer::load_voice_data( binistream *f, std::string const &bnk_filename
         }
 
         fp.close(bnk_file);
-        aud_vfs_fclose(fd);
+        vfs_fclose(fd);
 
         return true;
     }
@@ -668,7 +668,7 @@ int CrolPlayer::load_rol_instrument( binistream *f, SBnkHeader const &header, st
     else
     {
         // set up default instrument data here
-        memset( &usedIns.instrument, 0, kSizeofDataRecord );
+        memset( &usedIns.instrument, 0, sizeof(usedIns.instrument) );
     }
     ins_list.push_back( usedIns );
 
