@@ -1,6 +1,6 @@
 /*
 *
-* Author: Giacomo Lozito <james@develia.org>, (C) 2005-2006
+* Author: Giacomo Lozito <james@develia.org>, (C) 2005-2007
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
@@ -18,20 +18,78 @@
 *
 */
 
+#include <audacious/drct.h>
+#include <audacious/plugin.h>
+#include <libaudcore/eventqueue.h>
+
 #include "amidi-plug.h"
 
+InputPlugin *amidiplug_iplist[] = { &amidiplug_ip, NULL };
 
-InputPlugin *get_iplugin_info(void)
+SIMPLE_INPUT_PLUGIN (amidi-plug, amidiplug_iplist)
+
+static GMutex * init_mutex;
+static gboolean initted;
+
+static GCond * control_cond;
+static gint seek_time;
+
+static GThread * audio_thread;
+static GMutex * audio_control_mutex;
+static GCond * audio_control_cond;
+static gboolean audio_stop_flag, audio_pause_flag;
+static gint audio_seek_time;
+
+static void amidiplug_init (void)
 {
-  amidiplug_ip.description = g_strdup_printf(N_("AMIDI-Plug %s (MIDI Player)"), AMIDIPLUG_VERSION);
-  return &amidiplug_ip;
+    init_mutex = g_mutex_new ();
+    initted = FALSE;
 }
 
+static void soft_init (void)
+{
+    g_mutex_lock (init_mutex);
 
-static gboolean amidiplug_detect_by_content( gchar * filename , VFSFile * fp )
+    if (! initted)
+    {
+        amidiplug_gettime_mutex = g_mutex_new ();
+        amidiplug_playing_mutex = g_mutex_new ();
+        control_cond = g_cond_new ();
+
+        audio_control_mutex = g_mutex_new ();
+        audio_control_cond = g_cond_new ();
+
+        i_configure_cfg_ap_read ();
+        i_backend_load (amidiplug_cfg_ap.ap_seq_backend);
+
+        initted = TRUE;
+    }
+
+    g_mutex_unlock (init_mutex);
+}
+
+static void amidiplug_cleanup (void)
+{
+    if (initted)
+    {
+        i_backend_unload ();
+
+        g_mutex_free (amidiplug_gettime_mutex);
+        g_mutex_free (amidiplug_playing_mutex);
+        g_cond_free (control_cond);
+
+        g_mutex_free (audio_control_mutex);
+        g_cond_free (audio_control_cond);
+    }
+
+    g_mutex_free (init_mutex);
+}
+
+static gint amidiplug_is_our_file_from_vfs( const gchar *filename_uri , VFSFile *fp )
 {
   gchar magic_bytes[4];
-  gint res = 0;
+
+  soft_init ();
 
   if ( fp == NULL )
     return FALSE;
@@ -41,7 +99,7 @@ static gboolean amidiplug_detect_by_content( gchar * filename , VFSFile * fp )
 
   if ( !strncmp( magic_bytes , "MThd" , 4 ) )
   {
-    DEBUGMSG( "MIDI found, %s is a standard midi file\n" , filename );
+    DEBUGMSG( "MIDI found, %s is a standard midi file\n" , filename_uri );
     return TRUE;
   }
 
@@ -57,7 +115,7 @@ static gboolean amidiplug_detect_by_content( gchar * filename , VFSFile * fp )
 
     if ( !strncmp( magic_bytes , "RMID" , 4 ) )
     {
-      DEBUGMSG( "MIDI found, %s is a riff midi file\n" , filename );
+      DEBUGMSG( "MIDI found, %s is a riff midi file\n" , filename_uri );
       return TRUE;
     }
   }
@@ -65,55 +123,10 @@ static gboolean amidiplug_detect_by_content( gchar * filename , VFSFile * fp )
   return FALSE;
 }
 
-
-static gint amidiplug_is_our_file( gchar * filename )
+static void amidiplug_configure (void)
 {
-  VFSFile * fp;
-  gboolean result = FALSE;
-
-  fp = VFS_FOPEN( filename , "rb" );
-
-  if ( fp == NULL )
-    return FALSE;
-
-  result = amidiplug_detect_by_content( filename , fp );
-  VFS_FCLOSE( fp );
-
-  return result;
-}
-
-
-static gint amidiplug_is_our_file_from_vfs( gchar *filename , VFSFile *fp )
-{
-  return amidiplug_detect_by_content( filename , fp );
-}
-
-
-static void amidiplug_init( void )
-{
-  g_log_set_handler(NULL , G_LOG_LEVEL_WARNING , g_log_default_handler , NULL);
-  DEBUGMSG( "init, read configuration\n" );
-  /* read configuration for amidi-plug */
-  i_configure_cfg_ap_read();
-  amidiplug_playing_status = AMIDIPLUG_STOP;
-  backend.gmodule = NULL;
-  /* load the backend selected by user */
-  i_backend_load( amidiplug_cfg_ap.ap_seq_backend );
-}
-
-
-static void amidiplug_cleanup( void )
-{
-  i_backend_unload(); /* unload currently loaded backend */
-  g_free( amidiplug_ip.description );
-}
-
-
-static void amidiplug_configure( void )
-{
-  /* display the nice config dialog */
-  DEBUGMSG( "opening config system\n" );
-  i_configure_gui();
+    soft_init ();
+    i_configure_gui ();
 }
 
 
@@ -123,37 +136,24 @@ static void amidiplug_aboutbox( void )
 }
 
 
-static void amidiplug_file_info_box( gchar * filename )
+static void amidiplug_file_info_box (const gchar * filename)
 {
-  i_fileinfo_gui( filename );
+    soft_init ();
+    i_fileinfo_gui (filename);
 }
 
 
 static void amidiplug_stop( InputPlayback * playback )
 {
   DEBUGMSG( "STOP request at tick: %i\n" , midifile.playing_tick );
-  pthread_mutex_lock( &amidiplug_playing_mutex );
-  if (( amidiplug_playing_status == AMIDIPLUG_PLAY ) ||
-      ( amidiplug_playing_status == AMIDIPLUG_STOP ))
-  {
-    amidiplug_playing_status = AMIDIPLUG_STOP;
-    pthread_mutex_unlock( &amidiplug_playing_mutex );
-    pthread_join( amidiplug_play_thread , NULL );
-    if ( backend.autonomous_audio == FALSE )
-      pthread_join( amidiplug_audio_thread , NULL );
-    DEBUGMSG( "STOP activated (play thread joined)\n" );
-  }
-  else if ( amidiplug_playing_status == AMIDIPLUG_PAUSE )
-  {
-    amidiplug_playing_status = AMIDIPLUG_STOP;
-    DEBUGMSG( "STOP activated (from PAUSE to STOP)\n" );
-    pthread_mutex_unlock( &amidiplug_playing_mutex );
-  }
-  else /* AMIDIPLUG_ERR */
-  {
-    DEBUGMSG( "STOP activated (in error handling, ok)\n" );
-    pthread_mutex_unlock( &amidiplug_playing_mutex );
-  }
+
+  g_mutex_lock( amidiplug_playing_mutex );
+  amidiplug_playing_status = AMIDIPLUG_STOP;
+  g_cond_signal (control_cond);
+  g_mutex_unlock( amidiplug_playing_mutex );
+
+  g_thread_join (playback->thread);
+  playback->thread = NULL;
 
   /* kill the sequencer (while it may have been already killed if coming
      from pause, it's safe to do anyway since it checks for multiple calls) */
@@ -164,157 +164,69 @@ static void amidiplug_stop( InputPlayback * playback )
   if ( backend.gmodule != NULL )
     backend.seq_stop();
 
-  /* close audio if current backend works with output plugin */
-  if (( backend.gmodule != NULL ) && ( backend.autonomous_audio == FALSE ))
-  {
-    DEBUGMSG( "STOP activated, closing audio output plugin\n" );
-    playback->output->buffer_free();
-    playback->output->buffer_free();
-    playback->output->close_audio();
-  }
   /* free midi data (if it has not been freed yet) */
   i_midi_free( &midifile );
 }
 
-
-static void amidiplug_pause( InputPlayback * playback, gshort paused )
+static void amidiplug_pause (InputPlayback * playback, gshort paused)
 {
-  if ( paused )
-  {
-    DEBUGMSG( "PAUSE request at tick: %i\n" , midifile.playing_tick );
-    pthread_mutex_lock( &amidiplug_playing_mutex );
-    /* this cond is used to avoid race conditions */
-    while ( amidiplug_playing_status != AMIDIPLUG_PLAY )
-      pthread_cond_wait( &amidiplug_playing_cond , &amidiplug_playing_mutex );
-    amidiplug_playing_status = AMIDIPLUG_PAUSE;
-    pthread_mutex_unlock( &amidiplug_playing_mutex );
-
-    pthread_join( amidiplug_play_thread , NULL );
-    if ( backend.autonomous_audio == FALSE )
-      pthread_join( amidiplug_audio_thread , NULL );
-    DEBUGMSG( "PAUSE activated (play thread joined)\n" , midifile.playing_tick );
-
-    if ( backend.autonomous_audio == FALSE )
-      playback->output->pause(paused);
-
-    /* kill the sequencer */
-    backend.seq_off();
-  }
-  else
-  {
-    DEBUGMSG( "PAUSE deactivated, returning to tick %i\n" , midifile.playing_tick );
-    /* revive the sequencer */
-    backend.seq_on();
-    /* re-set initial tempo */
-    i_midi_setget_tempo( &midifile );
-    backend.seq_queue_tempo( midifile.current_tempo , midifile.ppq );
-    /* get back to the previous state */
-    amidiplug_skipto( midifile.playing_tick );
-
-    if ( backend.autonomous_audio == FALSE )
-      playback->output->pause(paused);
-
-    pthread_mutex_lock( &amidiplug_playing_mutex );
-    /* play play play! */
-    DEBUGMSG( "PAUSE deactivated, starting play thread again\n" );
-    pthread_create(&amidiplug_play_thread, NULL, amidiplug_play_loop, playback);
-    /* this cond is used to avoid race conditions */
-    while ( amidiplug_playing_status != AMIDIPLUG_PLAY )
-      pthread_cond_wait( &amidiplug_playing_cond , &amidiplug_playing_mutex );
-    pthread_mutex_unlock( &amidiplug_playing_mutex );
-  }
+    g_mutex_lock (amidiplug_playing_mutex);
+    amidiplug_playing_status = paused ? AMIDIPLUG_PAUSE : AMIDIPLUG_PLAY;
+    g_cond_signal (control_cond);
+    g_cond_wait (control_cond, amidiplug_playing_mutex);
+    g_mutex_unlock (amidiplug_playing_mutex);
 }
 
-
-static void amidiplug_seek( InputPlayback * playback, gint time )
+static void amidiplug_mseek (InputPlayback * playback, gulong time)
 {
-  DEBUGMSG( "SEEK requested (time %i), pausing song...\n" , time );
-  pthread_mutex_lock( &amidiplug_playing_mutex );
-  /* this cond is used to avoid race conditions */
-  while ( amidiplug_playing_status != AMIDIPLUG_PLAY )
-    pthread_cond_wait( &amidiplug_playing_cond , &amidiplug_playing_mutex );
-  amidiplug_playing_status = AMIDIPLUG_PAUSE;
-  pthread_mutex_unlock( &amidiplug_playing_mutex );
-
-  pthread_join( amidiplug_play_thread , NULL );
-  if ( backend.autonomous_audio == FALSE )
-    pthread_join( amidiplug_audio_thread , NULL );
-  DEBUGMSG( "SEEK requested (time %i), song paused\n" , time );
-  /* kill the sequencer */
-  backend.seq_off();
-  /* revive the sequencer */
-  backend.seq_on();
-  /* re-set initial tempo */
-  i_midi_setget_tempo( &midifile );
-  backend.seq_queue_tempo( midifile.current_tempo , midifile.ppq );
-  /* get back to the previous state */
-  DEBUGMSG( "SEEK requested (time %i), moving to tick %i of %i\n" ,
-            time , (gint)((time * 1000000) / midifile.avg_microsec_per_tick) , midifile.max_tick );
-  midifile.playing_tick = (gint)((time * 1000000) / midifile.avg_microsec_per_tick);
-  amidiplug_skipto( midifile.playing_tick );
-
-  if ( backend.autonomous_audio == FALSE )
-    playback->output->flush(time * 1000);
-
-  /* play play play! */
-  DEBUGMSG( "SEEK done, starting play thread again\n" );
-  pthread_create(&amidiplug_play_thread, NULL, amidiplug_play_loop, playback);
+    g_mutex_lock (amidiplug_playing_mutex);
+    amidiplug_playing_status = AMIDIPLUG_SEEK;
+    seek_time = time;
+    g_cond_signal (control_cond);
+    g_cond_wait (control_cond, amidiplug_playing_mutex);
+    g_mutex_unlock (amidiplug_playing_mutex);
 }
 
-
-static gint amidiplug_get_time( InputPlayback *playback )
+static gint amidiplug_get_time( InputPlayback * playback )
 {
-  if ( backend.autonomous_audio == FALSE )
-  {
-    pthread_mutex_lock( &amidiplug_playing_mutex );
+    gint pt;
+
+    if (! backend.autonomous_audio)
+        return -1;
+
+    g_mutex_lock (amidiplug_playing_mutex);
+
+    if (seek_time != -1)
+    {
+        gint time = seek_time;
+
+        g_mutex_unlock (amidiplug_playing_mutex);
+        return time;
+    }
+
     if (( amidiplug_playing_status == AMIDIPLUG_PLAY ) ||
         ( amidiplug_playing_status == AMIDIPLUG_PAUSE ) ||
-        (( amidiplug_playing_status == AMIDIPLUG_STOP ) && ( playback->output->buffer_playing() )))
+        ( amidiplug_playing_status == AMIDIPLUG_SEEK ))
     {
-      pthread_mutex_unlock( &amidiplug_playing_mutex );
-      return playback->output->output_time();
-    }
-    else if ( amidiplug_playing_status == AMIDIPLUG_STOP )
-    {
-      pthread_mutex_unlock( &amidiplug_playing_mutex );
-      DEBUGMSG( "GETTIME on stopped song, returning -1\n" , time );
-      return -1;
-    }
-    else /* AMIDIPLUG_ERR */
-    {
-      pthread_mutex_unlock( &amidiplug_playing_mutex );
-      DEBUGMSG( "GETTIME on halted song (an error occurred?), returning -1 and stopping the player\n" );
-      xmms_remote_stop(0);
-      return -1;
-    }
-  }
-  else
-  {
-    gint pt;
-    pthread_mutex_lock( &amidiplug_playing_mutex );
-    if (( amidiplug_playing_status == AMIDIPLUG_PLAY ) ||
-        ( amidiplug_playing_status == AMIDIPLUG_PAUSE ))
-    {
-      pthread_mutex_unlock( &amidiplug_playing_mutex );
-      pthread_mutex_lock(&amidiplug_gettime_mutex);
+      g_mutex_unlock( amidiplug_playing_mutex );
+      g_mutex_lock( amidiplug_gettime_mutex );
       pt = midifile.playing_tick;
-      pthread_mutex_unlock(&amidiplug_gettime_mutex);
+      g_mutex_unlock( amidiplug_gettime_mutex );
       return (gint)((pt * midifile.avg_microsec_per_tick) / 1000);
     }
     else if ( amidiplug_playing_status == AMIDIPLUG_STOP )
     {
-      pthread_mutex_unlock( &amidiplug_playing_mutex );
-      DEBUGMSG( "GETTIME on stopped song, returning -1\n" , time );
+      g_mutex_unlock( amidiplug_playing_mutex );
+      DEBUGMSG( "GETTIME on stopped song, returning -1\n" );
       return -1;
     }
     else /* AMIDIPLUG_ERR */
     {
-      pthread_mutex_unlock( &amidiplug_playing_mutex );
-      DEBUGMSG( "GETTIME on halted song (an error occurred?), returning -1 and stopping the player\n" , time );
-      xmms_remote_stop(0);
+      g_mutex_unlock( amidiplug_playing_mutex );
+      DEBUGMSG( "GETTIME on halted song (an error occurred?), returning -1 and stopping the player\n" );
+      aud_drct_stop();
       return -1;
     }
-  }
 }
 
 
@@ -340,46 +252,51 @@ static gint amidiplug_set_volume( gint  l , gint  r )
 }
 
 
-static void amidiplug_get_song_info( gchar * filename , gchar ** title , gint * length )
+static Tuple * amidiplug_get_song_tuple( const gchar *filename_uri )
 {
   /* song title, get it from the filename */
-  *title = G_PATH_GET_BASENAME(filename);
+  Tuple *tuple = tuple_new_from_filename(filename_uri);
+  gchar *title, *filename = g_filename_from_uri(filename_uri, NULL, NULL);
+  midifile_t mf;
 
-  /* sure, it's possible to calculate the length of a MIDI file anytime,
-     but the file must be entirely parsed to calculate it; this could
-     lead to a bit of performance loss, so let the user decide here */
-  if ( amidiplug_cfg_ap.ap_opts_length_precalc )
-  {
-    /* let's calculate the midi length, using this nice helper function that
-       will return 0 if a problem occurs and the length can't be calculated */
-    midifile_t mf;
+  soft_init ();
 
-    if ( i_midi_parse_from_filename( filename , &mf ) )
-      *length = (gint)(mf.length / 1000);
-    else
-      *length = -1;
-
-    i_midi_free( &mf );
-  }
+  if (filename != NULL)
+      title = g_path_get_basename(filename);
   else
-    *length = -1;
+      title = g_strdup(filename_uri);
 
-  return;
+  tuple_associate_string(tuple, FIELD_TITLE, NULL, title);
+  g_free(title);
+  g_free(filename);
+
+  if ( i_midi_parse_from_filename( filename_uri , &mf ) )
+    tuple_associate_int(tuple, FIELD_LENGTH, NULL, mf.length / 1000);
+
+  i_midi_free( &mf );
+
+  return tuple;
 }
 
 
-static void amidiplug_play( InputPlayback * playback)
+static void amidiplug_play( InputPlayback * playback )
 {
-  gchar * filename = playback->filename;
+  gchar * filename_uri = playback->filename;
+  gchar * filename = NULL;
   gint port_count = 0;
   gint au_samplerate = -1, au_bitdepth = -1, au_channels = -1;
+  Tuple *tu;
+
+  soft_init ();
 
   if ( backend.gmodule == NULL )
   {
     g_warning( "No sequencer backend selected\n" );
+    /* not usable, cause now amidiplug_play is in a different thread
     i_message_gui( _("AMIDI-Plug - warning") ,
                    _("No sequencer backend has been selected!\nPlease configure AMIDI-Plug before playing.") ,
                    AMIDIPLUG_MESSAGE_WARN , NULL , TRUE );
+    */
     amidiplug_playing_status = AMIDIPLUG_ERR;
     return;
   }
@@ -403,20 +320,22 @@ static void amidiplug_play( InputPlayback * playback)
   port_count = backend.seq_get_port_count();
   if ( port_count < 1 )
   {
-    g_warning( "No ports selected\n" );
+    event_queue ("interface show error", _("You have not selected any "
+     "sequencer ports for MIDI playback.  You can do so in the MIDI plugin "
+     "preferences."));
     amidiplug_playing_status = AMIDIPLUG_ERR;
     return;
   }
 
-  DEBUGMSG( "PLAY requested, opening file: %s\n" , filename );
-  midifile.file_pointer = VFS_FOPEN( filename , "rb" );
+  DEBUGMSG( "PLAY requested, opening file: %s\n" , filename_uri );
+  midifile.file_pointer = VFS_FOPEN( filename_uri , "rb" );
   if (!midifile.file_pointer)
   {
-    g_warning( "Cannot open %s\n" , filename );
+    g_warning( "Cannot open %s\n" , filename_uri );
     amidiplug_playing_status = AMIDIPLUG_ERR;
     return;
   }
-  midifile.file_name = filename;
+  midifile.file_name = g_strdup(filename_uri);
 
   switch( i_midi_file_read_id( &midifile ) )
   {
@@ -425,7 +344,7 @@ static void amidiplug_play( InputPlayback * playback)
       DEBUGMSG( "PLAY requested, RIFF chunk found, processing...\n" );
       /* read riff chunk */
       if ( !i_midi_file_parse_riff( &midifile ) )
-        WARNANDBREAKANDPLAYERR( "%s: invalid file format (riff parser)\n" , filename );
+        WARNANDBREAKANDPLAYERR( "%s: invalid file format (riff parser)\n" , filename_uri );
 
       /* if that was read correctly, go ahead and read smf data */
     }
@@ -434,97 +353,218 @@ static void amidiplug_play( InputPlayback * playback)
     {
       DEBUGMSG( "PLAY requested, MThd chunk found, processing...\n" );
       if ( !i_midi_file_parse_smf( &midifile , port_count ) )
-        WARNANDBREAKANDPLAYERR( "%s: invalid file format (smf parser)\n" , filename );
+        WARNANDBREAKANDPLAYERR( "%s: invalid file format (smf parser)\n" , filename_uri );
 
       if ( midifile.time_division < 1 )
-        WARNANDBREAKANDPLAYERR( "%s: invalid time division (%i)\n" , filename , midifile.time_division );
+        WARNANDBREAKANDPLAYERR( "%s: invalid time division (%i)\n" , filename_uri , midifile.time_division );
 
       DEBUGMSG( "PLAY requested, setting ppq and tempo...\n" );
       /* fill midifile.ppq and midifile.tempo using time_division */
       if ( !i_midi_setget_tempo( &midifile ) )
-        WARNANDBREAKANDPLAYERR( "%s: invalid values while setting ppq and tempo\n" , filename );
+        WARNANDBREAKANDPLAYERR( "%s: invalid values while setting ppq and tempo\n" , filename_uri );
 
       DEBUGMSG( "PLAY requested, sequencer start\n" );
       /* sequencer start */
-      if ( !backend.seq_start( filename ) )
-        WARNANDBREAKANDPLAYERR( "%s: problem with seq_start, play aborted\n" , filename );
+      if ( !backend.seq_start( filename_uri ) )
+        WARNANDBREAKANDPLAYERR( "%s: problem with seq_start, play aborted\n" , filename_uri );
 
       DEBUGMSG( "PLAY requested, sequencer on\n" );
       /* sequencer on */
       if ( !backend.seq_on() )
-        WARNANDBREAKANDPLAYERR( "%s: problem with seq_on, play aborted\n" , filename );
+        WARNANDBREAKANDPLAYERR( "%s: problem with seq_on, play aborted\n" , filename_uri );
 
       DEBUGMSG( "PLAY requested, setting sequencer queue tempo...\n" );
       /* set sequencer queue tempo using ppq and tempo (call only after i_midi_setget_tempo) */
       if ( !backend.seq_queue_tempo( midifile.current_tempo , midifile.ppq ) )
       {
         backend.seq_off(); /* kill the sequencer */
-        WARNANDBREAKANDPLAYERR( "%s: ALSA queue problem, play aborted\n" , filename );
+        WARNANDBREAKANDPLAYERR( "%s: ALSA queue problem, play aborted\n" , filename_uri );
       }
 
       /* fill midifile.length, keeping in count tempo-changes */
       i_midi_setget_length( &midifile );
       DEBUGMSG( "PLAY requested, song length calculated: %i msec\n" , (gint)(midifile.length / 1000) );
 
+
       /* our length is in microseconds, but the player wants milliseconds */
-      amidiplug_ip.set_info( G_PATH_GET_BASENAME(filename) ,
-                             (gint)(midifile.length / 1000) ,
+      filename = g_filename_from_uri( filename_uri , NULL , NULL );
+      if ( !filename ) filename = g_strdup( filename_uri );
+      tu = amidiplug_get_song_tuple(filename_uri);
+
+      playback->set_tuple( playback , tu );
+      playback->set_params( playback , NULL , 0 ,
                              au_bitdepth * au_samplerate * au_channels / 8 ,
                              au_samplerate , au_channels );
+      g_free( filename );
+
+      /* done with file */
+      VFS_FCLOSE( midifile.file_pointer );
+      midifile.file_pointer = NULL;
 
       /* play play play! */
       DEBUGMSG( "PLAY requested, starting play thread\n" );
+      g_mutex_lock( amidiplug_playing_mutex );
       amidiplug_playing_status = AMIDIPLUG_PLAY;
-      pthread_create(&amidiplug_play_thread, NULL, amidiplug_play_loop, playback);
+      g_mutex_unlock( amidiplug_playing_mutex );
+
+      seek_time = -1;
+      playback->playing = TRUE;
+      playback->set_pb_ready(playback);
+      amidiplug_play_loop(playback);
       break;
     }
 
     default:
     {
       amidiplug_playing_status = AMIDIPLUG_ERR;
-      g_warning( "%s is not a Standard MIDI File\n" , filename );
+      g_warning( "%s is not a Standard MIDI File\n" , filename_uri );
       break;
     }
   }
 
-  VFS_FCLOSE( midifile.file_pointer );
+  if (! backend.autonomous_audio)
+      playback->output->close_audio ();
+
+  if ( midifile.file_pointer )
+  {
+    /* done with file */
+    VFS_FCLOSE( midifile.file_pointer );
+    midifile.file_pointer = NULL;
+  }
+}
+
+static void * audio_loop (void * arg)
+{
+    InputPlayback * playback = arg;
+    gboolean paused = FALSE;
+    void * buffer = NULL;
+    gint buffer_size = 0;
+
+    while (1)
+    {
+        g_mutex_lock (audio_control_mutex);
+
+        if (audio_stop_flag)
+        {
+            g_cond_signal (audio_control_cond);
+            g_mutex_unlock (audio_control_mutex);
+            break;
+        }
+
+        if (audio_seek_time != -1)
+        {
+            playback->output->flush (audio_seek_time);
+            audio_seek_time = -1;
+            g_cond_signal (audio_control_cond);
+        }
+
+        if (audio_pause_flag)
+        {
+            if (! paused)
+            {
+                playback->output->pause (TRUE);
+                paused = TRUE;
+            }
+
+            g_cond_signal (audio_control_cond);
+            g_cond_wait (audio_control_cond, audio_control_mutex);
+            g_mutex_unlock (audio_control_mutex);
+            continue;
+        }
+
+        if (paused)
+        {
+            playback->output->pause (FALSE);
+            paused = FALSE;
+            g_cond_signal (audio_control_cond);
+        }
+
+        g_mutex_unlock (audio_control_mutex);
+
+        if (backend.seq_output (& buffer, & buffer_size))
+            playback->pass_audio (playback, FMT_S16_NE, 2, buffer_size, buffer,
+             NULL);
+    }
+
+    g_free (buffer);
+    return NULL;
+}
+
+static void audio_start (InputPlayback * playback)
+{
+    audio_stop_flag = FALSE;
+    audio_pause_flag = FALSE;
+    audio_seek_time = -1;
+    audio_thread = g_thread_create (audio_loop, playback, TRUE, NULL);
+}
+
+static void audio_seek (gint time)
+{
+    g_mutex_lock (audio_control_mutex);
+    audio_seek_time = time;
+    g_cond_signal (audio_control_cond);
+    g_cond_wait (audio_control_cond, audio_control_mutex);
+    g_mutex_unlock (audio_control_mutex);
+}
+
+static void audio_pause (gboolean pause)
+{
+    g_mutex_lock (audio_control_mutex);
+    audio_pause_flag = pause;
+    g_cond_signal (audio_control_cond);
+    g_cond_wait (audio_control_cond, audio_control_mutex);
+    g_mutex_unlock (audio_control_mutex);
+}
+
+static void audio_stop (void)
+{
+    g_mutex_lock (audio_control_mutex);
+    audio_stop_flag = TRUE;
+    g_cond_signal (audio_control_cond);
+    g_mutex_unlock (audio_control_mutex);
+    g_thread_join (audio_thread);
+}
+
+static void do_seek (gint time)
+{
+    backend.seq_event_allnoteoff (midifile.playing_tick);
+    backend.seq_queue_stop ();
+    amidiplug_skipto (time * (gint64) 1000 / midifile.avg_microsec_per_tick);
+}
+
+static void do_pause (gboolean pause)
+{
+    if (pause)
+    {
+        backend.seq_event_allnoteoff (midifile.playing_tick);
+        backend.seq_queue_stop ();
+    }
+    else
+    {
+        midifile.skip_offset = midifile.playing_tick;
+        backend.seq_queue_start ();
+    }
 }
 
 
-
-void * amidiplug_play_loop( void * arg )
+gpointer amidiplug_play_loop( gpointer arg )
 {
   InputPlayback *playback = arg;
-  gint i = 0;
-  gboolean rewind = FALSE;
-
-  pthread_mutex_lock( &amidiplug_playing_mutex );
-  if ( amidiplug_playing_status != AMIDIPLUG_PAUSE )
-  {
-    DEBUGMSG( "PLAY thread, rewind tracks to their first event\n" );
-    rewind = TRUE;
-  }
-  else
-  {
-    DEBUGMSG( "PLAY thread, do not rewind tracks to their first event (coming from a PAUSE status)\n" );
-    amidiplug_playing_status = AMIDIPLUG_PLAY;
-    pthread_cond_signal( &amidiplug_playing_cond );
-  }
-  pthread_mutex_unlock( &amidiplug_playing_mutex );
+  gint j = 0;
+  gboolean rewind = TRUE, paused = FALSE, stopped = FALSE;
 
   if ( rewind )
   {
     /* initialize current position in each track */
-    for (i = 0; i < midifile.num_tracks; ++i)
-      midifile.tracks[i].current_event = midifile.tracks[i].first_event;
-    backend.seq_queue_start();
+    for (j = 0; j < midifile.num_tracks; ++j)
+      midifile.tracks[j].current_event = midifile.tracks[j].first_event;
   }
 
-  if ( backend.autonomous_audio == FALSE )
-  {
-    pthread_create(&amidiplug_audio_thread, NULL, amidiplug_audio_loop, playback);
-  }
+  if (! backend.autonomous_audio)
+      audio_start (playback);
 
+  /* queue start */
+  backend.seq_queue_start();
   /* common settings for all our events */
   backend.seq_event_init();
 
@@ -535,6 +575,68 @@ void * amidiplug_play_loop( void * arg )
     midifile_track_t * event_track = NULL;
     gint i, min_tick = midifile.max_tick + 1;
 
+    g_mutex_lock (amidiplug_playing_mutex);
+
+    if (amidiplug_playing_status == AMIDIPLUG_STOP)
+    {
+        g_mutex_unlock (amidiplug_playing_mutex);
+        stopped = TRUE;
+        break;
+    }
+
+    if (amidiplug_playing_status == AMIDIPLUG_SEEK)
+    {
+        if (! backend.autonomous_audio)
+            audio_seek (seek_time);
+
+        /* WORKAROUND: wait till unpause to seek */
+        if (! paused)
+        {
+            do_seek (seek_time);
+            seek_time = -1;
+        }
+
+        amidiplug_playing_status = paused ? AMIDIPLUG_PAUSE : AMIDIPLUG_PLAY;
+        g_cond_signal (control_cond);
+    }
+
+    if (amidiplug_playing_status == AMIDIPLUG_PAUSE)
+    {
+        if (! paused)
+        {
+            do_pause (TRUE);
+
+            if (! backend.autonomous_audio)
+                audio_pause (TRUE);
+
+            paused = TRUE;
+            g_cond_signal (control_cond);
+        }
+
+        g_mutex_unlock (amidiplug_playing_mutex);
+        continue;
+    }
+
+    if (paused)
+    {
+        do_pause (FALSE);
+
+        if (! backend.autonomous_audio)
+            audio_pause (FALSE);
+
+        /* WORKAROUND (see above) */
+        if (seek_time != -1)
+        {
+            do_seek (seek_time);
+            seek_time = -1;
+        }
+
+        paused = FALSE;
+        g_cond_signal (control_cond);
+    }
+
+    g_mutex_unlock (amidiplug_playing_mutex);
+
     /* search next event */
     for (i = 0; i < midifile.num_tracks; ++i)
     {
@@ -548,15 +650,6 @@ void * amidiplug_play_loop( void * arg )
       }
     }
 
-    /* check if the song has been stopped */
-    pthread_mutex_lock( &amidiplug_playing_mutex );
-    if ( amidiplug_playing_status != AMIDIPLUG_PLAY )
-    {
-      DEBUGMSG( "PLAY thread, PAUSE or STOP requested, exiting from play loop\n" );
-      event = NULL;
-    }
-    pthread_mutex_unlock( &amidiplug_playing_mutex );
-
     if (!event)
       break; /* end of song reached */
 
@@ -596,9 +689,9 @@ void * amidiplug_play_loop( void * arg )
         backend.seq_event_tempo( event );
         DEBUGMSG( "PLAY thread, processing tempo event with value %i on tick %i\n" ,
                   event->data.tempo , event->tick );
-        pthread_mutex_lock(&amidiplug_gettime_mutex);
+        g_mutex_lock( amidiplug_gettime_mutex );
         midifile.current_tempo = event->data.tempo;
-        pthread_mutex_unlock(&amidiplug_gettime_mutex);
+        g_mutex_unlock( amidiplug_gettime_mutex );
         break;
       case SND_SEQ_EVENT_META_TEXT:
         /* do nothing */
@@ -611,9 +704,9 @@ void * amidiplug_play_loop( void * arg )
         break;
     }
 
-    pthread_mutex_lock(&amidiplug_gettime_mutex);
+    g_mutex_lock( amidiplug_gettime_mutex );
     midifile.playing_tick = event->tick;
-    pthread_mutex_unlock(&amidiplug_gettime_mutex);
+    g_mutex_unlock( amidiplug_gettime_mutex );
 
     if ( backend.autonomous_audio == TRUE )
     {
@@ -622,17 +715,21 @@ void * amidiplug_play_loop( void * arg )
     }
   }
 
-  backend.seq_output_shut( midifile.max_tick , midifile.skip_offset );
+  backend.seq_output_shut (stopped ? midifile.playing_tick : midifile.max_tick,
+   midifile.skip_offset);
 
-  pthread_mutex_lock( &amidiplug_playing_mutex );
-  if ( amidiplug_playing_status != AMIDIPLUG_PAUSE )
+  if (! backend.autonomous_audio)
   {
-    amidiplug_playing_status = AMIDIPLUG_STOP;
-    DEBUGMSG( "PLAY thread, song stopped/ended\n" );
-  }
-  pthread_mutex_unlock( &amidiplug_playing_mutex );
+      audio_stop ();
 
-  pthread_exit(NULL);
+      if (! stopped)
+      {
+          while (playback->output->buffer_playing ())
+              g_usleep (20000);
+      }
+  }
+
+  return NULL;
 }
 
 
@@ -721,9 +818,9 @@ void amidiplug_skipto( gint playing_tick )
         break;
       case SND_SEQ_EVENT_TEMPO:
         backend.seq_event_tempo( event );
-        pthread_mutex_lock(&amidiplug_gettime_mutex);
+        g_mutex_lock( amidiplug_gettime_mutex );
         midifile.current_tempo = event->data.tempo;
-        pthread_mutex_unlock(&amidiplug_gettime_mutex);
+        g_mutex_unlock( amidiplug_gettime_mutex );
         break;
     }
 
@@ -738,29 +835,3 @@ void amidiplug_skipto( gint playing_tick )
 
   return;
 }
-
-
-void * amidiplug_audio_loop( void * arg )
-{
-  InputPlayback *playback = arg;
-  gboolean going = 1;
-  gpointer buffer = NULL;
-  gint buffer_size = 0;
-  while ( going )
-  {
-    if ( backend.seq_output( &buffer , &buffer_size ) )
-    {
-      while( ( playback->output->buffer_free() < buffer_size ) && ( going == TRUE ) )
-        G_USLEEP(10000);
-      produce_audio( playback->output->written_time() ,
-                     FMT_S16_NE , 2 , buffer_size , buffer , &going );
-    }
-    pthread_mutex_lock( &amidiplug_playing_mutex );
-    if ( amidiplug_playing_status != AMIDIPLUG_PLAY )
-      going = FALSE;
-    pthread_mutex_unlock( &amidiplug_playing_mutex );
-  }
-  if ( buffer != NULL )
-    g_free( buffer );
-  pthread_exit(NULL);
-}
