@@ -36,9 +36,16 @@
 
 #include <glib.h>
 
-#include <audacious/plugin.h>
+#include <audacious/audconfig.h>
+#include <audacious/configdb.h>
+#include <audacious/debug.h>
 #include <audacious/i18n.h>
-#include <audacious/output.h>
+#include <audacious/misc.h>
+#include <audacious/playlist.h>
+#include <audacious/plugin.h>
+#include <libaudcore/eventqueue.h>
+#include <libaudgui/libaudgui.h>
+#include <libaudgui/libaudgui-gtk.h>
 
 #include "cdaudio-ng.h"
 #include "configure.h"
@@ -137,7 +144,7 @@ static void cdaudio_error (const gchar * message_format, ...)
     msg = g_markup_vprintf_escaped (message_format, args);
     va_end (args);
 
-    aud_event_queue_with_data_free ("interface show error", msg);
+    event_queue_with_data_free ("interface show error", msg);
 }
 
 /* main thread only */
@@ -202,17 +209,11 @@ static void cdaudio_init ()
     cdng_cfg.use_dae = TRUE;
     cdng_cfg.use_cdtext = TRUE;
     cdng_cfg.use_cddb = TRUE;
-    cdng_cfg.device = g_strdup ("");
-    cdng_cfg.cddb_server = g_strdup (CDDA_DEFAULT_CDDB_SERVER);
-    cdng_cfg.cddb_path = g_strdup ("");
     cdng_cfg.cddb_port = CDDA_DEFAULT_CDDB_PORT;
     cdng_cfg.cddb_http = FALSE;
     cdng_cfg.disc_speed = DEFAULT_DISC_SPEED;
     cdng_cfg.use_proxy = FALSE;
-    cdng_cfg.proxy_host = g_strdup ("");
     cdng_cfg.proxy_port = CDDA_DEFAULT_PROXY_PORT;
-    cdng_cfg.proxy_username = g_strdup ("");
-    cdng_cfg.proxy_password = g_strdup ("");
 
     if ((db = aud_cfg_db_open ()) == NULL)
     {
@@ -239,6 +240,19 @@ static void cdaudio_init ()
     aud_cfg_db_get_string (db, "audacious", "proxy_pass",
                            &cdng_cfg.proxy_password);
 
+    if (cdng_cfg.device == NULL)
+        cdng_cfg.device = g_strdup ("");
+    if (cdng_cfg.cddb_server == NULL)
+        cdng_cfg.cddb_server = g_strdup (CDDA_DEFAULT_CDDB_SERVER);
+    if (cdng_cfg.cddb_path == NULL)
+        cdng_cfg.cddb_path = g_strdup ("");
+    if (cdng_cfg.proxy_host == NULL)
+        cdng_cfg.proxy_host = g_strdup ("");
+    if (cdng_cfg.proxy_username == NULL)
+        cdng_cfg.proxy_username = g_strdup ("");
+    if (cdng_cfg.proxy_password == NULL)
+        cdng_cfg.proxy_password = g_strdup ("");
+
     aud_cfg_db_close (db);
 
     if (!cdio_init ())
@@ -256,29 +270,18 @@ static void cdaudio_init ()
 }
 
 /* main thread only */
-static void cdaudio_about ()
+static void cdaudio_about (void)
 {
-    static GtkWidget *about_window = NULL;
+    static GtkWidget * about_window = NULL;
 
-    if (about_window)
-    {
-        gtk_window_present (GTK_WINDOW (about_window));
-    }
-    else
-    {
-        about_window = audacious_info_dialog (_("About Audio CD Plugin"),
-                                              _
-                                              ("Copyright (c) 2007, by Calin Crisan <ccrisan@gmail.com> and The Audacious Team.\n\n"
-                                               "Many thanks to libcdio developers <http://www.gnu.org/software/libcdio/>\n"
-                                               "\tand to libcddb developers <http://libcddb.sourceforge.net/>.\n\n"
-                                               "Also thank you Tony Vroon for mentoring & guiding me.\n\n"
-                                               "This was a Google Summer of Code 2007 project.\n\n"
-                                               "Copyright 2009 John Lindgren"),
-                                              _("Close"), FALSE, NULL, NULL);
-
-        g_signal_connect (G_OBJECT (about_window), "destroy",
-                          G_CALLBACK (gtk_widget_destroyed), &about_window);
-    }
+    audgui_simple_message (& about_window, GTK_MESSAGE_INFO,
+     _("About Audio CD Plugin"),
+     _("Copyright (c) 2007, by Calin Crisan <ccrisan@gmail.com> and The Audacious Team.\n\n"
+     "Many thanks to libcdio developers <http://www.gnu.org/software/libcdio/>\n"
+     "\tand to libcddb developers <http://libcddb.sourceforge.net/>.\n\n"
+     "Also thank you Tony Vroon for mentoring & guiding me.\n\n"
+     "This was a Google Summer of Code 2007 project.\n\n"
+     "Copyright 2009 John Lindgren"));
 }
 
 /* main thread only */
@@ -603,6 +606,13 @@ static void cdaudio_cleanup (void)
     aud_cfg_db_set_string (db, "CDDA", "device", cdng_cfg.device);
     aud_cfg_db_close (db);
 
+    g_free (cdng_cfg.device);
+    g_free (cdng_cfg.cddb_server);
+    g_free (cdng_cfg.cddb_path);
+    g_free (cdng_cfg.proxy_host);
+    g_free (cdng_cfg.proxy_username);
+    g_free (cdng_cfg.proxy_password);
+
     g_mutex_unlock (mutex);
     g_mutex_free (mutex);
     g_cond_free (control_cond);
@@ -627,7 +637,7 @@ static Tuple *create_tuple_from_trackinfo_and_filename (const gchar * filename)
 
     if (!strcmp (filename, "cdda://"))
     {
-        tuple = aud_tuple_new_from_filename (filename);
+        tuple = tuple_new_from_filename (filename);
         tuple->nsubtunes = 1 + lasttrackno - firsttrackno;
         tuple->subtunes = g_malloc (sizeof *tuple->subtunes * tuple->nsubtunes);
 
@@ -645,27 +655,27 @@ static Tuple *create_tuple_from_trackinfo_and_filename (const gchar * filename)
         goto DONE;
     }
 
-    tuple = aud_tuple_new_from_filename (filename);
+    tuple = tuple_new_from_filename (filename);
 
     if (strlen (trackinfo[trackno].performer))
     {
-        aud_tuple_associate_string (tuple, FIELD_ARTIST, NULL,
+        tuple_associate_string (tuple, FIELD_ARTIST, NULL,
                                     trackinfo[trackno].performer);
     }
     if (strlen (trackinfo[0].name))
     {
-        aud_tuple_associate_string (tuple, FIELD_ALBUM, NULL,
+        tuple_associate_string (tuple, FIELD_ALBUM, NULL,
                                     trackinfo[0].name);
     }
     if (strlen (trackinfo[trackno].name))
     {
-        aud_tuple_associate_string (tuple, FIELD_TITLE, NULL,
+        tuple_associate_string (tuple, FIELD_TITLE, NULL,
                                     trackinfo[trackno].name);
     }
 
-    aud_tuple_associate_int (tuple, FIELD_TRACK_NUMBER, NULL, trackno);
+    tuple_associate_int (tuple, FIELD_TRACK_NUMBER, NULL, trackno);
 
-    aud_tuple_associate_int (tuple, FIELD_LENGTH, NULL,
+    tuple_associate_int (tuple, FIELD_LENGTH, NULL,
                              calculate_track_length (trackinfo[trackno].
                                                      startlsn,
                                                      trackinfo[trackno].
@@ -673,7 +683,7 @@ static Tuple *create_tuple_from_trackinfo_and_filename (const gchar * filename)
 
     if (strlen (trackinfo[trackno].genre))
     {
-        aud_tuple_associate_string (tuple, FIELD_GENRE, NULL,
+        tuple_associate_string (tuple, FIELD_GENRE, NULL,
                                     trackinfo[trackno].genre);
     }
 
