@@ -9,24 +9,15 @@
 #include <unistd.h>
 #include <math.h>
 
+extern "C" {
+#include <audacious/configdb.h>
+}
+
 #include "modplugbmp.h"
 #include "stdafx.h"
 #include "sndfile.h"
 #include "stddefs.h"
 #include "archive/open.h"
-extern "C" {
-#include <audacious/output.h>
-}
-
-static char* format_and_free_ti( Tuple* ti, int* length )
-{
-        char* result = aud_tuple_formatter_make_title_string(ti, aud_get_gentitle_format());
-        if ( result )
-                *length = aud_tuple_get_int(ti, FIELD_LENGTH, NULL);
-        aud_tuple_free((void *) ti);
-
-        return result;
-}
 
 // ModplugXMMS member functions ===============================
 
@@ -34,11 +25,14 @@ static char* format_and_free_ti( Tuple* ti, int* length )
 ModplugXMMS::ModplugXMMS()
 {
 	mSoundFile = new CSoundFile;
-	mOutPlug = NULL;
+    control_mutex = g_mutex_new ();
+    control_cond = g_cond_new ();
 }
 ModplugXMMS::~ModplugXMMS()
 {
 	delete mSoundFile;
+    g_mutex_free (control_mutex);
+    g_cond_free (control_cond);
 }
 
 ModplugXMMS::Settings::Settings()
@@ -64,17 +58,17 @@ ModplugXMMS::Settings::Settings()
 	mBassRange      = 30;
 	mSurroundDepth  = 20;
 	mSurroundDelay  = 20;
-	
+
 	mPreamp         = false;
 	mPreampLevel    = 0.0f;
-	
+
 	mLoopCount      = 0;   //don't loop
 }
 
 void ModplugXMMS::Init(void)
 {
 	mcs_handle_t *db;
-	
+
 	db = aud_cfg_db_open();
 
 	aud_cfg_db_get_bool(db, MODPLUG_CFGID,"Surround", &mModProps.mSurround);
@@ -110,7 +104,7 @@ bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
 	const int magicSize = 32;
 	char magic[magicSize];
 
-	aud_vfs_fread(magic, 1, magicSize, file);
+	vfs_fread(magic, 1, magicSize, file);
 	if (!memcmp(magic, UMX_MAGIC, 4))
 		return true;
 	if (!memcmp(magic, "Extended Module:", 16))
@@ -124,14 +118,14 @@ bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
 	if (!memcmp(magic, PSM_MAGIC, 4))
 		return true;
 
-	aud_vfs_fseek(file, 44, SEEK_SET);
-	aud_vfs_fread(magic, 1, 4, file);
+	vfs_fseek(file, 44, SEEK_SET);
+	vfs_fread(magic, 1, 4, file);
 	if (!memcmp(magic, S3M_MAGIC, 4))
 		return true;
 
-	aud_vfs_fseek(file, 1080, SEEK_SET);
-	aud_vfs_fread(magic, 1, 4, file);
-	
+	vfs_fseek(file, 1080, SEEK_SET);
+	vfs_fread(magic, 1, 4, file);
+
 	// Check for Fast Tracker multichannel modules (xCHN, xxCH)
 	if (magic[1] == 'C' && magic[2] == 'H' && magic[3] == 'N') {
 		if (magic[0] == '6' || magic[0] == '8')
@@ -142,7 +136,7 @@ bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
 		if ((nch % 2 == 0) && nch >= 10)
 			return true;
 	}
-	
+
 	// Check for Amiga MOD module formats
 	if(mModProps.mGrabAmigaMOD) {
 	if (!memcmp(magic, MOD_MAGIC_PROTRACKER4, 4))
@@ -187,6 +181,8 @@ bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
 		return true;
 	if (lExt == ".dbf")
 		return true;
+	if (lExt == ".dmf")
+		return true;
 	if (lExt == ".dsm")
 		return true;
 	if (lExt == ".far")
@@ -200,69 +196,61 @@ bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
 	if (lExt == ".mt2")
 		return true;
 
-	if (lExt == ".mdz")
-		return true;
-	if (lExt == ".mdr")
-		return true;
-	if (lExt == ".mdgz")
-		return true;
-	if (lExt == ".mdbz")
-		return true;
-	if (lExt == ".s3z")
-		return true;
-	if (lExt == ".s3r")
-		return true;
-	if (lExt == ".s3gz")
-		return true;
-	if (lExt == ".xmz")
-		return true;
-	if (lExt == ".xmr")
-		return true;
-	if (lExt == ".xmgz")
-		return true;
-	if (lExt == ".itz")
-		return true;
-	if (lExt == ".itr")
-		return true;
-	if (lExt == ".itgz")
-		return true;
-	if (lExt == ".dmf")
-		return true;
-	
-	if (lExt == ".zip")
-		return ContainsMod(aFilename);
-	if (lExt == ".gz")
-		return ContainsMod(aFilename);
-	if (lExt == ".bz2")
-		return ContainsMod(aFilename);
-	if (lExt == ".rar")
-		return ContainsMod(aFilename);
-	if (lExt == ".rb")
-		return ContainsMod(aFilename);
-
 	return false;
 }
 
 void ModplugXMMS::PlayLoop(InputPlayback *playback)
 {
 	uint32 lLength;
-	//the user might change the number of channels while playing.
-	// we don't want this to take effect until we are done!
-	uint8 lChannels = mModProps.mChannels;
+    gboolean paused = FALSE;
+
+    g_mutex_lock (control_mutex);
+    seek_time = -1;
+    mPaused = FALSE;
+    playback->playing = TRUE;
+    playback->set_pb_ready (playback);
+    g_mutex_unlock (control_mutex);
+
+    while (1)
+    {
+        g_mutex_lock (control_mutex);
+
+        if (! playback->playing)
+        {
+            g_mutex_unlock (control_mutex);
+            break;
+        }
+
+        if (seek_time != -1)
+        {
+            mSoundFile->SetCurrentPos (seek_time * (gint64)
+             mSoundFile->GetMaxPosition () / (mSoundFile->GetSongTime () * 1000));
+            playback->output->flush (seek_time);
+            seek_time = -1;
+            g_cond_signal (control_cond);
+        }
+
+        if (mPaused != paused)
+        {
+            playback->output->pause (mPaused);
+            paused = mPaused;
+            g_cond_signal (control_cond);
+        }
+
+        if (paused)
+        {
+            g_cond_wait (control_cond, control_mutex);
+            g_mutex_unlock (control_mutex);
+            continue;
+        }
+
+        g_mutex_unlock (control_mutex);
+
+        lLength = mSoundFile->Read (mBuffer, mBufSize);
+
+        if (! lLength)
+            break;
 
-	while(!mStopped)
-	{
-		if(!(lLength = mSoundFile->Read(
-				mBuffer,
-				mBufSize)))
-		{
-			//no more to play.  Wait for output to finish and then stop.
-			while((mOutPlug->buffer_playing())
-			   && (!mStopped))
-				usleep(10000);
-			break;
-		}
-		
 		if(mModProps.mPreamp)
 		{
 			//apply preamp
@@ -273,10 +261,10 @@ void ModplugXMMS::PlayLoop(InputPlayback *playback)
 					short old = ((short*)mBuffer)[i];
 					((short*)mBuffer)[i] *= (short int)mPreampFactor;
 					// detect overflow and clip!
-					if ((old & 0x8000) != 
+					if ((old & 0x8000) !=
 					 (((short*)mBuffer)[i] & 0x8000))
 					  ((short*)mBuffer)[i] = old | 0x7FFF;
-						
+
 				}
 			}
 			else
@@ -285,40 +273,24 @@ void ModplugXMMS::PlayLoop(InputPlayback *playback)
 					uchar old = ((uchar*)mBuffer)[i];
 					((uchar*)mBuffer)[i] *= (short int)mPreampFactor;
 					// detect overflow and clip!
-					if ((old & 0x80) != 
+					if ((old & 0x80) !=
 					 (((uchar*)mBuffer)[i] & 0x80))
 					  ((uchar*)mBuffer)[i] = old | 0x7F;
 				}
 			}
 		}
-		
-		if(mStopped)
-			break;
-	
-		//wait for buffer space to free up.
-		while(((mOutPlug->buffer_free()
-		    < (int)mBufSize))
-		   && (!mStopped))
-			usleep(10000);
-			
-		if(mStopped)
-			break;
-		
-		playback->pass_audio
-		(
-			playback,
-			mFormat,
-			lChannels,
-			mBufSize,
-			mBuffer,
-			NULL
-		);
 
-		mPlayed += mBufTime;
+        playback->output->write_audio (mBuffer, mBufSize);
 	}
 
-//	mOutPlug->flush(0);
-	mOutPlug->close_audio();
+    g_mutex_lock (control_mutex);
+
+    while (playback->playing && playback->output->buffer_playing ())
+        g_usleep (10000);
+
+    playback->playing = FALSE;
+    g_cond_signal (control_cond); /* wake up any waiting request */
+    g_mutex_unlock (control_mutex);
 
 	//Unload the file
 	mSoundFile->Destroy();
@@ -329,29 +301,21 @@ void ModplugXMMS::PlayLoop(InputPlayback *playback)
 		delete [] mBuffer;
 		mBuffer = NULL;
 	}
-
-	mPaused = false;
-	mStopped = true;
 }
 
-void ModplugXMMS::PlayFile(const string& aFilename, InputPlayback *ipb)
+bool ModplugXMMS::PlayFile(const string& aFilename, InputPlayback *ipb)
 {
-	int32 aLength=0;
-	char *aModName=NULL;
-	mStopped = true;
-	mPaused = false;
-	
 	//open and mmap the file
 	mArchive = OpenArchive(aFilename);
 	if(mArchive->Size() == 0)
 	{
 		delete mArchive;
-		return;
+		return true;
 	}
-	
+
 	if (mBuffer)
 		delete [] mBuffer;
-	
+
 	//find buftime to get approx. 512 samples/block
 	mBufTime = 512000 / mModProps.mFrequency + 1;
 
@@ -363,7 +327,7 @@ void ModplugXMMS::PlayFile(const string& aFilename, InputPlayback *ipb)
 
 	mBuffer = new uchar[mBufSize];
 	if(!mBuffer)
-		return;             //out of memory!
+		return true;        //out of memory!
 
 	CSoundFile::SetWaveConfig
 	(
@@ -381,7 +345,7 @@ void ModplugXMMS::PlayFile(const string& aFilename, InputPlayback *ipb)
 		mModProps.mNoiseReduction,
 		false
 	);
-	
+
 	// [Reverb level 0(quiet)-100(loud)], [delay in ms, usually 40-200ms]
 	if(mModProps.mReverb)
 	{
@@ -412,96 +376,88 @@ void ModplugXMMS::PlayFile(const string& aFilename, InputPlayback *ipb)
 	CSoundFile::SetResamplingMode(mModProps.mResamplingMode);
 	mSoundFile->SetRepeatCount(mModProps.mLoopCount);
 	mPreampFactor = exp(mModProps.mPreampLevel);
-	
-	mPaused = false;
-	mStopped = false;
 
 	mSoundFile->Create
 	(
 		(uchar*)mArchive->Map(),
 		mArchive->Size()
 	);
-	mPlayed = 0;
 
-        Tuple* ti = GetSongTuple( aFilename );
-        if ( ti )
-                aModName = format_and_free_ti( ti, &aLength );
+    Tuple* ti = GetSongTuple( aFilename );
+    if ( ti ) {
+        ipb->set_tuple(ipb,ti);
+    }
 
 	ipb->set_params
 	(
 		ipb,
-		aModName,
-		aLength,
+		NULL,
+		0,
 		mSoundFile->GetNumChannels() * 1000,
 		mModProps.mFrequency,
 		mModProps.mChannels
 	);
 
-	mStopped = mPaused = false;
-
 	if(mModProps.mBits == 16)
 		mFormat = FMT_S16_NE;
 	else
 		mFormat = FMT_U8;
 
-	mOutPlug->open_audio
-	(
-		mFormat,
-		mModProps.mFrequency,
-		mModProps.mChannels
-	);
+    if (! ipb->output->open_audio (mFormat, mModProps.mFrequency,
+     mModProps.mChannels))
+    {
+        ipb->error = TRUE;
+        return true;
+    }
 
-	mDecodeThread = g_thread_self();
-
-	ipb->playing = TRUE;
-	ipb->set_pb_ready(ipb);
 	this->PlayLoop(ipb);
-	ipb->playing = FALSE;
+	ipb->output->close_audio ();
+
+    return false;
 }
 
-void ModplugXMMS::Stop(void)
+void ModplugXMMS::Stop (InputPlayback * playback)
 {
-	if(mStopped)
-		return;
+    g_mutex_lock (control_mutex);
 
-	mStopped = true;
-	mPaused = false;
-	
-	g_thread_join(mDecodeThread);
+    if (playback->playing)
+    {
+        playback->playing = FALSE;
+        g_cond_signal (control_cond);
+        g_mutex_unlock (control_mutex);
+        g_thread_join (playback->thread);
+        playback->thread = NULL;
+    }
+    else
+        g_mutex_unlock (control_mutex);
 }
 
-void ModplugXMMS::Pause(bool aPaused)
+void ModplugXMMS::pause (InputPlayback * playback, gshort paused)
 {
-	if(aPaused)
-		mPaused = true;
-	else
-		mPaused = false;
-	
-	mOutPlug->pause(aPaused);
+    g_mutex_lock (control_mutex);
+
+    if (playback->playing)
+    {
+        mPaused = paused;
+        g_cond_signal (control_cond);
+        g_cond_wait (control_cond, control_mutex);
+    }
+
+    g_mutex_unlock (control_mutex);
 }
 
-void ModplugXMMS::Seek(float32 aTime)
+void ModplugXMMS::mseek (InputPlayback * playback, gulong time)
 {
-	uint32  lMax;
-	uint32  lMaxtime;
-	float32 lPostime;
-	
-	if(aTime > (lMaxtime = mSoundFile->GetSongTime()))
-		aTime = lMaxtime;
-	lMax = mSoundFile->GetMaxPosition();
-	lPostime = float(lMax) / lMaxtime;
+    g_mutex_lock (control_mutex);
 
-	mSoundFile->SetCurrentPos(int(aTime * lPostime));
+    if (playback->playing)
+    {
+        seek_time = time;
+        g_cond_signal (control_cond);
+        g_cond_wait (control_cond, control_mutex);
+    }
 
-	mOutPlug->flush(int(aTime * 1000));
-	mPlayed = uint32(aTime * 1000);
-}
-
-float32 ModplugXMMS::GetTime(void)
-{
-	if ((mStopped) || (!mOutPlug))
-		return -1;
-	return (float32)mOutPlug->output_time() / 1000;
+    g_mutex_unlock (control_mutex);
 }
 
 Tuple* ModplugXMMS::GetSongTuple(const string& aFilename)
@@ -509,7 +465,7 @@ Tuple* ModplugXMMS::GetSongTuple(const string& aFilename)
 	CSoundFile* lSoundFile;
 	Archive* lArchive;
 	const gchar *tmps;
-	
+
 	//open and mmap the file
         lArchive = OpenArchive(aFilename);
         if(lArchive->Size() == 0)
@@ -518,10 +474,10 @@ Tuple* ModplugXMMS::GetSongTuple(const string& aFilename)
                 return NULL;
         }
 
-	Tuple *ti = aud_tuple_new_from_filename(aFilename.c_str());
+	Tuple *ti = tuple_new_from_filename(aFilename.c_str());
 	lSoundFile = new CSoundFile;
 	lSoundFile->Create((uchar*)lArchive->Map(), lArchive->Size());
-	
+
 	switch(lSoundFile->GetType())
         {
 	case MOD_TYPE_MOD:	tmps = "ProTracker"; break;
@@ -547,17 +503,17 @@ Tuple* ModplugXMMS::GetSongTuple(const string& aFilename)
 	case MOD_TYPE_PSM:	tmps = "Protracker Studio Module"; break;
 	default:		tmps = "ModPlug unknown"; break;
 	}
-	aud_tuple_associate_string(ti, FIELD_CODEC, NULL, tmps);
-	aud_tuple_associate_string(ti, FIELD_QUALITY, NULL, "sequenced");
-	aud_tuple_associate_int(ti, FIELD_LENGTH, NULL, lSoundFile->GetSongTime() * 1000);
-	
+	tuple_associate_string(ti, FIELD_CODEC, NULL, tmps);
+	tuple_associate_string(ti, FIELD_QUALITY, NULL, "sequenced");
+	tuple_associate_int(ti, FIELD_LENGTH, NULL, lSoundFile->GetSongTime() * 1000);
+
 	gchar *tmps2 = MODPLUG_CONVERT(lSoundFile->GetTitle());
 	// Chop any leading spaces off. They are annoying in the playlist.
 	gchar *tmps3 = tmps2; // Make another pointer so tmps2 can still be free()d
 	while ( *tmps3 == ' ' ) tmps3++ ;
-	aud_tuple_associate_string(ti, FIELD_TITLE, NULL, tmps3);
+	tuple_associate_string(ti, FIELD_TITLE, NULL, tmps3);
 	g_free(tmps2);
-	
+
 	//unload the file
 	lSoundFile->Destroy();
 	delete lSoundFile;
@@ -569,10 +525,6 @@ void ModplugXMMS::SetInputPlugin(InputPlugin& aInPlugin)
 {
 	mInPlug = &aInPlugin;
 }
-void ModplugXMMS::SetOutputPlugin(OutputPlugin& aOutPlugin)
-{
-	mOutPlug = &aOutPlugin;
-}
 
 const ModplugXMMS::Settings& ModplugXMMS::GetModProps()
 {
