@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 /* As a special exception, when this file is copied by Bison into a
    Bison output file, you may use that output file without restriction.
@@ -1380,7 +1380,7 @@ int yylex (YYSTYPE *yylval, void *yyparam) {
   parser_control *pc = (parser_control *) yyparam;
   
   /* Ignore whitespace, get first nonwhite character. */
-  while ((c = aud_vfs_getc (pc->input)) == ' ' || c == '\t' || c == '\n');
+  while ((c = vfs_getc (pc->input)) == ' ' || c == '\t' || c == '\n');
   
   /* End of input ? */
   if (c == EOF)
@@ -1388,7 +1388,7 @@ int yylex (YYSTYPE *yylval, void *yyparam) {
 
   /* Char starts a number => parse the number. */
   if (isdigit (c)) {
-    aud_vfs_fseek (pc->input, -1, SEEK_CUR); /* Put the char back. */
+    vfs_fseek (pc->input, -1, SEEK_CUR); /* Put the char back. */
     {
       char *old_locale, *saved_locale;
 
@@ -1399,10 +1399,10 @@ int yylex (YYSTYPE *yylval, void *yyparam) {
 
       while (isdigit(c) || c == '.')
       {
-        c = aud_vfs_getc(pc->input);
+        c = vfs_getc(pc->input);
       }
 
-      aud_vfs_fseek(pc->input, -1, SEEK_CUR);
+      vfs_fseek(pc->input, -1, SEEK_CUR);
 
       setlocale (LC_ALL, saved_locale);
       g_free (saved_locale);
@@ -1420,10 +1420,10 @@ int yylex (YYSTYPE *yylval, void *yyparam) {
       sym_name = g_string_append_c (sym_name, c);
 
       /* Get another character. */
-      c = aud_vfs_getc (pc->input);
+      c = vfs_getc (pc->input);
     } while (c != EOF && isalnum (c));
     
-    aud_vfs_fseek (pc->input, -1, SEEK_CUR);
+    vfs_fseek (pc->input, -1, SEEK_CUR);
 
     yylval->s_value = sym_name->str;
     
@@ -1504,7 +1504,7 @@ expression_t *expr_compile_string (const char* str, symbol_dict_t *dict)
 
   g_return_val_if_fail(str != NULL && dict != NULL, NULL);
 
-  stream = aud_vfs_buffer_new_from_string ( (char *) str );
+  stream = vfs_buffer_new_from_string ( (char *) str );
 
   pc.input = stream;
   pc.expr = expr_new ();
@@ -1516,7 +1516,7 @@ expression_t *expr_compile_string (const char* str, symbol_dict_t *dict)
     pc.expr = NULL;
   }
 
-  aud_vfs_fclose (stream);
+  vfs_fclose (stream);
 
   return pc.expr;
 }
