@@ -27,10 +27,14 @@
 #include <limits.h>
 
 #include <gtk/gtk.h>
+#include <pulse/pulseaudio.h>
+
+#include <audacious/debug.h>
+#include <audacious/drct.h>
 #include <audacious/plugin.h>
 #include <audacious/i18n.h>
-
-#include <pulse/pulseaudio.h>
+#include <libaudgui/libaudgui.h>
+#include <libaudgui/libaudgui-gtk.h>
 
 #define ERROR(...) do {fprintf (stderr, "pulseaudio: " __VA_ARGS__); putchar ('\n');} while (0)
 
@@ -65,15 +69,15 @@ do { \
     if (!connected) return retval; \
 } while (0);
 
-static const char* get_song_name(void) {
-    static char t[256];
-    char *str, *u;
-
-    if (!(str = aud_playback_get_title()))
-        return "Playback Stream";
-    snprintf(t, sizeof(t), "%s", u = pa_locale_to_utf8(str));
-    pa_xfree(u);
+static const gchar * get_song_name (void)
+{
+    if (! aud_drct_get_playing ()) /* just probing? */
+        return "";
 
+    gchar * title = aud_drct_get_title ();
+    static gchar t[512];
+    snprintf (t, sizeof (t), "%s", title);
+    g_free (title);
     return t;
 }
 
@@ -507,7 +511,7 @@ static void pulse_close(void)
     volume_time_event = NULL;
 }
 
-static int pulse_open(AFormat fmt, int rate, int nch) {
+static int pulse_open(gint fmt, int rate, int nch) {
     pa_sample_spec ss;
     pa_operation *o = NULL;
     int success;
@@ -683,11 +687,7 @@ static OutputPluginInitStatus pulse_init (void)
 
 static void pulse_about(void) {
     static GtkWidget *dialog;
-
-    if (dialog != NULL)
-        return;
-
-    dialog = audacious_info_dialog(
+    audgui_simple_message(& dialog, GTK_MESSAGE_INFO,
             _("About Audacious PulseAudio Output Plugin"),
             _("Audacious PulseAudio Output Plugin\n\n "
             "This program is free software; you can redistribute it and/or modify\n"
@@ -703,17 +703,7 @@ static void pulse_about(void) {
             "You should have received a copy of the GNU General Public License\n"
             "along with this program; if not, write to the Free Software\n"
             "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,\n"
-            "USA."),
-            _("OK"),
-            FALSE,
-            NULL,
-            NULL);
-
-    gtk_signal_connect(
-            GTK_OBJECT(dialog),
-            "destroy",
-            GTK_SIGNAL_FUNC(gtk_widget_destroyed),
-            &dialog);
+            "USA."));
 }
 
 static OutputPlugin pulse_op = {
