@@ -9,17 +9,17 @@
 #include <stdio.h>
 
 #include <glib.h>
-#include <audacious/plugin.h>
-#include <audacious/audutil.h>
-#include <audacious/i18n.h>
-
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
-#include "configure.h"
+#include <audacious/configdb.h>
+#include <audacious/i18n.h>
+#include <audacious/preferences.h>
+#include <libaudcore/md5.h>
+
 #include "plugin.h"
 
-GtkWidget *entry1, *entry2, *entry3, *ge_entry1, *ge_entry2, *cfgdlg;
+GtkWidget *entry1, *entry2, *entry3, *cfgdlg;
 static GdkColor disabled_color;
 guint apply_timeout = 0; /* ID of timeout to save new config */
 gboolean running = TRUE; /* if plugin threads are running */
@@ -42,7 +42,6 @@ static char *hexify(char *pass, int len)
 }
 
 static char *pwd = NULL;
-static char *ge_pwd = NULL;
 
 static void saveconfig(void)
 {
@@ -50,44 +49,29 @@ static void saveconfig(void)
 
     const char *uid = gtk_entry_get_text(GTK_ENTRY(entry1));
     const char *url = gtk_entry_get_text(GTK_ENTRY(entry3));
-    const char *ge_uid = gtk_entry_get_text(GTK_ENTRY(ge_entry1));
 
     if ((cfgfile = aud_cfg_db_open())) {
         aud_md5state_t md5state;
-        unsigned char md5pword[16], ge_md5pword[16];
+        unsigned char md5pword[16];
 
-        if (uid != NULL && uid[0] != '\0' &&
-            pwd != NULL && pwd[0] != '\0')
-        {
-            aud_cfg_db_set_string(cfgfile, "audioscrobbler", "username", (char *)uid);
+        if (pwd != NULL && pwd[0] != '\0') {
             aud_md5_init(&md5state);
-            aud_md5_append(&md5state, (unsigned const char *)pwd, strlen(pwd));
+            aud_md5_append(&md5state, (guchar *)pwd, strlen(pwd));
             aud_md5_finish(&md5state, md5pword);
             aud_cfg_db_set_string(cfgfile, "audioscrobbler", "password",
-                                 hexify((char*)md5pword, sizeof(md5pword)));
-        } else if (!uid || uid[0] == '\0') {
+                                 hexify((gchar*)md5pword, sizeof(md5pword)));
+        }
+        if (uid != NULL && uid[0] != '\0') {
+            aud_cfg_db_set_string(cfgfile, "audioscrobbler", "username", (gchar *)uid);
+        } else {
             aud_cfg_db_set_string(cfgfile, "audioscrobbler", "username", "");
             aud_cfg_db_set_string(cfgfile, "audioscrobbler", "password", "");
         }
-        
-        if (url != NULL && url[0] != '\0')
-        	aud_cfg_db_set_string(cfgfile, "audioscrobbler", "sc_url", (char *)url);
-       	else if (!url || url[0] == '\0')
-       		aud_cfg_db_set_string(cfgfile, "audioscrobbler", "sc_url", LASTFM_HS_URL);
 
-        if (ge_uid != NULL && ge_uid[0] != '\0' &&
-            ge_pwd != NULL && ge_pwd[0] != '\0')
-        {
-            aud_cfg_db_set_string(cfgfile, "audioscrobbler", "ge_username", (char *)ge_uid);
-            aud_md5_init(&md5state);
-            aud_md5_append(&md5state, (unsigned const char *)ge_pwd, strlen(ge_pwd));
-            aud_md5_finish(&md5state, ge_md5pword);
-            aud_cfg_db_set_string(cfgfile, "audioscrobbler", "ge_password",
-                                  hexify((char*)ge_md5pword, sizeof(ge_md5pword)));
-        } else if (!ge_uid || ge_uid[0] == '\0') {
-            aud_cfg_db_set_string(cfgfile, "audioscrobbler", "ge_username", "");
-            aud_cfg_db_set_string(cfgfile, "audioscrobbler", "ge_password", "");
-        }
+        if (url != NULL && url[0] != '\0')
+        	aud_cfg_db_set_string(cfgfile, "audioscrobbler", "sc_url", (gchar *)url);
+       	else
+       		aud_cfg_db_set_string(cfgfile, "audioscrobbler", "sc_url", LASTFM_HS_URL);
 
         aud_cfg_db_close(cfgfile);
     }
@@ -101,14 +85,16 @@ static gboolean apply_config_changes(gpointer data) {
     return FALSE;
 }
 
-void configure_cleanup(void) {
+static void configure_apply(void) {
     if (apply_timeout) { /* config has been changed, but wasn't saved yet */
         g_source_remove(apply_timeout);
-        apply_timeout = 0;
-        saveconfig();
+        apply_config_changes(NULL);
     }
+}
+
+static void configure_cleanup(void) {
     g_free(pwd);
-    g_free(ge_pwd);
+    pwd = NULL;
 }
 
 static void
@@ -139,11 +125,7 @@ static void entry_focus_out(GtkWidget *widget, gpointer data)
     g_free(pwd);
     pwd = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry2)));
   }
-  if (widget == ge_entry2)
-  {
-    g_free(ge_pwd);
-    ge_pwd = g_strdup(gtk_entry_get_text(GTK_ENTRY(ge_entry2)));
-  }
+
   entry_changed(widget, data);
   gtk_entry_set_text(GTK_ENTRY(widget), _("Change password"));
   gtk_widget_modify_text(widget, GTK_STATE_NORMAL, &disabled_color);
@@ -151,7 +133,7 @@ static void entry_focus_out(GtkWidget *widget, gpointer data)
 }
 
 /* Generated by glade, sorta. */
-GtkWidget *
+static void *
 create_cfgdlg(void)
 {
   mcs_handle_t *db;
@@ -161,7 +143,6 @@ create_cfgdlg(void)
   GtkWidget *label1;
   GtkWidget *label2;
   GtkWidget *label4;
-  GtkWidget *himage1;
   GtkWidget *align1;
   GtkWidget *notebook1;
   GtkStyle *style;
@@ -231,7 +212,7 @@ create_cfgdlg(void)
                    NULL);
   gtk_widget_show (entry2);
   gtk_table_attach_defaults (GTK_TABLE (table1), entry2, 1, 2, 3, 4);
-  
+
   entry3 = gtk_entry_new ();
   gtk_widget_show (entry3);
   gtk_table_attach_defaults (GTK_TABLE (table1), entry3, 1, 2, 4, 5);
@@ -240,64 +221,10 @@ create_cfgdlg(void)
   gtk_label_set_use_markup (GTK_LABEL (label1), TRUE);
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook1), GTK_WIDGET(align1), label1);
 
-
-  // gerpok
-  align1 = gtk_alignment_new(0, 0, 0, 0);
-  gtk_widget_show (align1);
-  gtk_alignment_set_padding(GTK_ALIGNMENT(align1), 0, 0, 12, 0);
-
-  table1 = gtk_table_new (2, 2, FALSE);
-  gtk_widget_show (table1);
-  gtk_container_add(GTK_CONTAINER(align1), table1);
-  gtk_table_set_row_spacings (GTK_TABLE(table1), 6);
-  gtk_table_set_col_spacings (GTK_TABLE(table1), 6);
-
-  label2 = gtk_label_new (_("Username:"));
-  gtk_widget_show (label2);
-  gtk_table_attach_defaults (GTK_TABLE (table1), label2, 0, 1, 2, 3);
-  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_RIGHT);
-  gtk_misc_set_alignment (GTK_MISC (label2), 1, 0.5);
-
-  label3 = gtk_label_new (_("Password:"));
-  gtk_widget_show (label3);
-  gtk_table_attach (GTK_TABLE (table1), label3, 0, 1, 3, 4,
-                    (GtkAttachOptions) (GTK_FILL),
-                    (GtkAttachOptions) (0), 0, 0);
-  gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_RIGHT);
-  gtk_misc_set_alignment (GTK_MISC (label3), 1, 0.5);
-
-  ge_entry1 = gtk_entry_new ();
-  gtk_widget_show (ge_entry1);
-  gtk_table_attach_defaults (GTK_TABLE (table1), ge_entry1, 1, 2, 2, 3);
-
-  ge_entry2 = gtk_entry_new ();
-  gtk_widget_modify_text(ge_entry2, GTK_STATE_NORMAL, &disabled_color);
-
-  gtk_entry_set_text(GTK_ENTRY(ge_entry2), "Change password");
-  g_signal_connect(G_OBJECT(ge_entry2), "focus-in-event",
-                   G_CALLBACK(entry_focus_in),
-                   NULL);
-  g_signal_connect(G_OBJECT(ge_entry2), "focus-out-event",
-                   G_CALLBACK(entry_focus_out),
-                   NULL);
-
-  gtk_widget_show (ge_entry2);
-  gtk_table_attach_defaults (GTK_TABLE (table1), ge_entry2, 1, 2, 3, 4);
-
-  label1 = gtk_label_new (_("<b>Gerpok</b>"));
-  gtk_label_set_use_markup (GTK_LABEL (label1), TRUE);
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook1), GTK_WIDGET(align1), label1);
-
   // common
   gtk_box_pack_start (GTK_BOX (vbox2), notebook1, TRUE, TRUE, 6);
 
-  himage1 = gtk_image_new_from_file (DATA_DIR "/images/audioscrobbler_badge.png");
-  gtk_widget_show (himage1);
-  gtk_box_pack_start (GTK_BOX (vbox2), himage1, FALSE, FALSE, 0);
-  gtk_misc_set_alignment (GTK_MISC (himage1), 1, 0.5);
-
 	gtk_entry_set_text(GTK_ENTRY(entry1), "");
-	gtk_entry_set_text(GTK_ENTRY(ge_entry1), "");
 
         if ((db = aud_cfg_db_open())) {
                 gchar *username = NULL;
@@ -310,30 +237,33 @@ create_cfgdlg(void)
                         g_free(username);
 			username = NULL;
                 }
-                
-				aud_cfg_db_get_string(db, "audioscrobbler", "sc_url", &sc_url);
-				if (sc_url) {
-                		gtk_entry_set_text(GTK_ENTRY(entry3), sc_url);
-                		g_free(sc_url);
-		             	sc_url = NULL;
-				}
-				
-		// gerpok
-                aud_cfg_db_get_string(db, "audioscrobbler", "ge_username",
-                        &username);
-                if (username) {
-                        gtk_entry_set_text(GTK_ENTRY(ge_entry1), username);
-                        g_free(username);
-			username = NULL;
-                }
+
+		aud_cfg_db_get_string(db, "audioscrobbler", "sc_url", &sc_url);
+		if (sc_url) {
+               		gtk_entry_set_text(GTK_ENTRY(entry3), sc_url);
+               		g_free(sc_url);
+	             	sc_url = NULL;
+		}
 
                 aud_cfg_db_close(db);
         }
 
   g_signal_connect(entry1, "changed", G_CALLBACK(entry_changed), NULL);
   g_signal_connect(entry3, "changed", G_CALLBACK(entry_changed), NULL);
-  g_signal_connect(ge_entry1, "changed", G_CALLBACK(entry_changed), NULL);
 
   return vbox2;
 }
 
+/* TODO: don't use WIDGET_CUSTOM there */
+static PreferencesWidget settings[] = {
+    {WIDGET_CUSTOM, NULL, NULL, NULL, NULL, FALSE, {.populate = create_cfgdlg}},
+};
+
+PluginPreferences preferences = {
+    .title = N_("Scrobbler"),
+    .prefs = settings,
+    .n_prefs = G_N_ELEMENTS(settings),
+    .apply = configure_apply,
+    .cleanup = configure_cleanup,
+};
+
